@@ -1,5 +1,5 @@
 /*
- * snappy - 0.1
+ * snappy - 0.2
  *
  * Copyright (C) 2011 Collabora Multimedia Ltd.
  * <luis.debethencourt@collabora.co.uk>
@@ -23,18 +23,24 @@
 #ifndef __GST_ENGINE_H__
 #define __GST_ENGINE_H__
 
+#include <gst/gst.h>
+
 G_BEGIN_DECLS
 
 typedef struct _GstEngine GstEngine;
 
 struct _GstEngine
 {
-  gboolean playing;
+  gboolean playing, direction_foward, prev_done;
+  gboolean has_started;
+  gboolean has_video, has_audio;
+  gboolean secret;
 
   guint media_width, media_height;
   gint64 media_duration;
+  gint64 second;
 
-  gchar *uri, *fileuri;
+  gchar *uri;
 
   GstElement *player;
   GstElement *sink;
@@ -43,7 +49,21 @@ struct _GstEngine
 };
 
 // Declaration of non-static functions
+gboolean add_uri_unfinished (GstEngine * engine);
+gboolean at_the_eos (GstEngine * engine);
 gboolean bus_call (GstBus * bus, GstMessage * msg, gpointer data);
+gboolean engine_init (GstEngine * engine, GstElement * sink);
+gboolean engine_load_uri (GstEngine * engine, gchar * uri);
+gboolean engine_open_uri (GstEngine * engine, gchar * uri);
+gboolean engine_play (GstEngine * engine);
+gboolean engine_seek (GstEngine * engine, gint64 position);
+gboolean engine_stop (GstEngine * engine);
+gboolean engine_volume (GstEngine * engine, gdouble level);
+gboolean frame_stepping (GstEngine * engine, gboolean foward);
+gchar **get_recently_viewed ();
+GstState get_state (GstEngine * engine);
+gint64 query_position (GstEngine * engine);
+gboolean change_state (GstEngine * engine, gchar * state);
 gboolean update_media_duration (GstEngine * engine);
 
 G_END_DECLS
